<?php /* _\|/_              vim: set et ts=4 sw=4 sts=4 nowrap fdm=marker:
         (o o)                         
+-----oOO-{_}-OOo--------------------------------------------------------+
| ConPresso - Foolproof Content Management                               |
+------------------------------------------------------------------------+
| Copyright (c) Bartels Schoene GmbH & Co. KG, 48149 Muenster, Germany   |
+------------------------------------------------------------------------+
| This source file is subject to ConPresso license version 2002-11-16    |
| that is bundled with this package in the files LICENSE_EN.TXT or       |
| LICENSE_DE.TXT, and is available through the world-wide-web at         |
|     http://www.conpresso.de/license/LICENSE_EN-2002-11-16.TXT or       |
|     http://www.conpresso.de/license/LICENSE_DE-2002-11-16.TXT          |
|                                                                        |
| If you did not receive a copy of the ConPresso license and are unable  |
| to obtain it through the world-wide-web, please send a note to         |
| <license@conpresso.de> so we can e-mail you a copy immediately.        |
+------------------------------------------------------------------------+
| Authors: ConPresso development and support team <support@conpresso.de> |
+------------------------------------------------------------------------+
  $Id$ */
define('CPO', true);
require_once('../_include/checkaccess.inc.php');

$modules = $activeModules; // needed because common-module.inc.php gets included once only
$d = dir(CPO_BASEDIR); 

while (false!==($entry = $d->read())) {
    if ($entry=='.' || $entry=='..') continue;
    $moduleDirectory = $entry;
    $entry = CPO_BASEDIR.$entry;
    if (!is_dir($entry) || !is_readable($entry)) continue;
    if (!file_exists($entry.'/common-module.inc.php') ||
        !is_readable($entry.'/common-module.inc.php')) continue;
    include_once($entry.'/common-module.inc.php');
}
$d->close();

// additionally load existing options
$db->query('SELECT name, options FROM '.CPO_MODULES);
while ($db->next_record()) {
    if (!is_string($db->v('options'))) continue;

    $options = unserialize($db->v('options'));
    if (!is_array($options)) continue;

    $modules[$db->v('name')] = $options;
}

// do the human readable sorting
$sortArray = array();
foreach($modules AS $mk=>$mv)  $sortArray[$mk] = (isset($modules[$mk]['caption'])) ? __($modules[$mk]['caption']) : $mk;
asort($sortArray);
$saveArray = $modules;
$modules = array();
foreach($sortArray AS $mk=>$mv) $modules[$mk] = $saveArray[$mk];

$views = array('list');

$actions = array();
if (isset($_REQUEST['action']))
    is_array($_REQUEST['action']) ? $actions = $_REQUEST['action'] : $actions = array($_REQUEST['action']);
while ($nextaction = array_shift($actions)) {
    switch ($nextaction) {
    case 'modules_install': // {{{
        if (empty($_REQUEST['module'])) break;

        doActions($modules[$_REQUEST['module']]['install']);

        $db = new DB;
        $query = "INSERT ".CPO_MODULES." (name, status, options) VALUES ("
                ."'".addslashes($_REQUEST['module'])."', "
                .MODULE_INACTIVE.', '
                ."'".addslashes(serialize($modules[$_REQUEST['module']]))."')";
        $db->query($query);
        
        logaction('install module', $_REQUEST['module']);

        //$views = array('');
        break; // }}}
    case 'modules_activate': // {{{
        if (empty($_REQUEST['module'])) break;

        $db = new DB;
        $query = "UPDATE ".CPO_MODULES." "
                ."SET status=".MODULE_ACTIVE." "
                ."WHERE name='".addslashes($_REQUEST['module'])."'";
        $db->query($query);

        $activeModules[$_REQUEST['module']] = $modules[$_REQUEST['module']];
        doActions($modules[$_REQUEST['module']]['activate']);

        logaction('activate module', $_REQUEST['module']);
        //$views = array('');
        break; // }}}
    case 'modules_deactivate': // {{{
        if (empty($_REQUEST['module'])) break;

        $db = new DB;
        $query = "UPDATE ".CPO_MODULES." "
                .'SET status='.MODULE_INACTIVE.' '
                ."WHERE name='".addslashes($_REQUEST['module'])."' ";
        $db->query($query);

        doActions($modules[$_REQUEST['module']]['deactivate']);
        $newActiveModules = array();
        foreach ($activeModules as $k=>$v) {
            if ($k==$_REQUEST['module']) continue;
            $newActiveModules[$k] = $v;
        }
        $activeModules = $newActiveModules;

        logaction('deactivate module', $_REQUEST['module']);
        //$views = array('');
        break; // }}}
     case 'modules_uninstall': // {{{
        $views = array('reallyuninstall');
        break; // }}}
    case 'modules_uninstall_really': // {{{
        if (empty($_REQUEST['module'])) break;

        if (!$antiCSRF->checkToken()) {
            $message[] = ___('Security check failed. Please try again.');
            $views = array('reallyuninstall');
            logaction('uninstall module', 'CSRF check failed.');
            break;
        }

        if (isset($_POST['no'])) {
            $message[] = sprintf(___('The module %s was not uninstalled.'),
                '<em>' . htmlspecialchars($modules[$_POST['module']]['caption'], ENT_COMPAT, 'ISO-8859-15').'</em>'
            );
            break;
        }

        doActions($modules[$_REQUEST['module']]['uninstall']);

        $db = new DB;
        $query = "DELETE FROM ".CPO_MODULES." "
                ."WHERE name='".addslashes($_REQUEST['module'])."' ";
        $db->query($query);

        logaction('uninstalled module', $_REQUEST['module']);
        //$views = array('');
        break; // }}}
    }
}

while ($nextview = array_shift($views)) {
    switch ($nextview) {
    case 'list': // {{{
        require(CPO_BASEDIR.'_include/header.php');

        printPageHeader(
            __('Work on modules'),
            __('In this view you can manage modules.'),
            'modules-workonmodules.html#modules-workonmodules'
        );

        if (count($modules)==0) {
            $message[] = ___('There are no installed modules.');
            include(CPO_BASEDIR.'_include/message.php');
            require(CPO_BASEDIR.'_include/footer.php');
            break;
        }

        include(CPO_BASEDIR.'_include/message.php');

        echo '<table class="ruler" style="width: 100%;">';
        echo '<thead><tr>';
        echo '<th style="width: 75%;"><span>'.___('Module').help('modules-workonmodules.html#modules-workonmodules-module').'</span></th>';
        echo '<th style="width: 25%;"><span>'.___('Manage').help('modules-workonmodules.html#modules-workonmodules-manage').'</span></th>';
        echo '</tr></thead><tbody>';

        $db = new DB;
        $db->query('SELECT name, status FROM '.CPO_MODULES);
        $module_status = array();
        while ($db->next_record()) {
            $module_status[$db->v('name')] = $db->v('status');
        }

        foreach ($modules as $k=>$v) {
            echo '<tr>';
            echo '<td>';
            echo (!array_key_exists($k, $module_status) || $module_status[$k]==MODULE_INACTIVE) ? '<span style="color:grey;">' : '';
            if (isset($v['caption']) && trim($v['caption'])!='') {
                echo ___($v['caption']);
            } else {
                echo htmlspecialchars($k, ENT_COMPAT, 'ISO-8859-15');
            }
            echo (array_key_exists($k, $module_status) && $module_status[$k]==MODULE_INACTIVE) ? ' ['.__('deactivated').']</span>' : '';
            echo (!array_key_exists($k, $module_status)) ? ' ['.__('not installed').']</span>' : '';
            echo '</td>';

            echo '<td>';
            printIcon(
                BASE_REL.'_admin/modules.php?action=modules_install&module='.urlencode($k), 
                SKIN_URL.'pic_mod_install.gif', 
                !array_key_exists($k, $module_status) ? __('install module') : __('The module is installed already.'), 
                !array_key_exists($k, $module_status)
            );
            printIcon(
                BASE_REL.'_admin/modules.php?action=modules_activate&module='.urlencode($k), 
                SKIN_URL.'pic_mod_active_on.gif', 
                !array_key_exists($k, $module_status) ? __('install module first') : ($module_status[$k]!=MODULE_ACTIVE ? __('activate module') : __('The module is activated already.')), 
                array_key_exists($k, $module_status) && $module_status[$k]!=MODULE_ACTIVE
            );
            printIcon(
                BASE_REL.'_admin/modules.php?action=modules_deactivate&module='.urlencode($k), 
                SKIN_URL.'pic_mod_active_off.gif', 
                !array_key_exists($k, $module_status) ? __('install module first') : ($module_status[$k]!=MODULE_INACTIVE ? __('deactivate module') : __('The module is deactivated.')), 
                array_key_exists($k, $module_status) && $module_status[$k]!=MODULE_INACTIVE
            );
            printIcon(
                BASE_REL.'_admin/modules.php?action=modules_uninstall&module='.urlencode($k), 
                SKIN_URL.'pic_del.gif', 
                array_key_exists($k, $module_status) && $module_status[$k]==MODULE_INACTIVE ? __('uninstall module') : __('The module can not be uninstalled.'), 
                array_key_exists($k, $module_status) && $module_status[$k]==MODULE_INACTIVE
            );

            echo '<img src="'.SKIN_URL.'pic_empty.gif" alt="" />';
            if (isset($modules[$k]['manage'])) doActions($modules[$k]['manage']);
            echo '</td>';
            echo '</tr>';
        }
        echo '</tbody></table>';

        //next_page($_SESSION['SID_pagers']['articles'], $max_data, $cpoSystem['noOfArticles'], 1, '', BASE_REL.'_admin/articles.php');
    
        require(CPO_BASEDIR.'_include/footer.php');
        break;
        // }}}
    case 'reallyuninstall': // {{{
        require(CPO_BASEDIR.'_include/header.php');

        printPageHeader(
            ___('Uninstall module'),
            sprintf(___('You are going to uninstall the module %s.'),
                '<em>' . htmlspecialchars($modules[$_GET['module']]['caption'], ENT_COMPAT, 'ISO-8859-15') . '</em>'
            ),
            '' // helpme
        );
        include(CPO_BASEDIR.'_include/message.php');

        $message = sprintf(___('Are you sure you want to uninstall the module %s?'),
            '<em>'.htmlspecialchars($modules[$_GET['module']]['caption'], ENT_COMPAT, 'ISO-8859-15').'</em>'
        );
        $message .= ' <br/> <br/>';

        $message .= '<form method="post" action="'.$_SERVER['PHP_SELF'].'">';
        $message .= $antiCSRF->getInput('modules_uninstall_really');
        $message .= '<input type="hidden" name="module" value="'.htmlspecialchars($_REQUEST['module'], ENT_COMPAT, 'ISO-8859-15').'" />';
//        $message .= '<input type="hidden" name="name" value="'.htmlspecialchars($name, ENT_COMPAT, 'ISO-8859-15').'" />';
        $message .= '<input type="submit" name="yes" value="'.___('Yes, uninstall').'" />&nbsp;';
        $message .= '<input type="submit" name="no" value="'.___('No, do not uninstall').'" />';
        $message .= '</form>';
        include(CPO_BASEDIR.'_include/confirmation.php');
        include(CPO_BASEDIR.'_include/footer.php');
        break; // }}}
    }
}
?>
