<?php /* _\|/_              vim: set et ts=4 sw=4 sts=4 nowrap fdm=marker:
         (o o)                         
+-----oOO-{_}-OOo--------------------------------------------------------+
| ConPresso - Foolproof Content Management                               |
+------------------------------------------------------------------------+
| Copyright (c) Bartels Schoene GmbH & Co. KG, 48149 Muenster, Germany   |
+------------------------------------------------------------------------+
| This source file is subject to ConPresso license version 2002-11-16    |
| that is bundled with this package in the files LICENSE_EN.TXT or       |
| LICENSE_DE.TXT, and is available through the world-wide-web at         |
|     http://www.conpresso.de/license/LICENSE_EN-2002-11-16.TXT or       |
|     http://www.conpresso.de/license/LICENSE_DE-2002-11-16.TXT          |
|                                                                        |
| If you did not receive a copy of the ConPresso license and are unable  |
| to obtain it through the world-wide-web, please send a note to         |
| <license@conpresso.de> so we can e-mail you a copy immediately.        |
+------------------------------------------------------------------------+
| Authors: ConPresso development and support team <support@conpresso.de> |
+------------------------------------------------------------------------+
  $Id$ */
define('CPO', true);
require_once('../_include/common.inc.php');

/*
 * If you need to disallow access to some files, you can use a .htaccess file
 * in the _data folder with the following content:
 *
 * RewriteEngine On
 * RewriteRule (secret_.*)$ index.php?file=$1
 * 
 * This rewrites all files starting with "secret_" to be delivered through this 
 * index.php file. 
 * 
 * The file then checks if a user is logged in before sending the requested 
 * file. But since no other access rights can be defined in ConPresso this is 
 * the only check that is performed.
 */

$mime_types = array(
    'txt' => 'text/plain',
	'htm' => 'text/html',
	'html' => 'text/html',
	'php' => 'text/html',
	'css' => 'text/css',
	'js' => 'application/javascript',
	'json' => 'application/json',
	'xml' => 'application/xml',
	'swf' => 'application/x-shockwave-flash',
	'flv' => 'video/x-flv',
	
	// images
	'png' => 'image/png',
	'jpe' => 'image/jpeg',
	'jpeg' => 'image/jpeg',
	'jpg' => 'image/jpeg',
	'gif' => 'image/gif',
	'bmp' => 'image/bmp',
	'ico' => 'image/vnd.microsoft.icon',
	'tiff' => 'image/tiff',
	'tif' => 'image/tiff',
	'svg' => 'image/svg+xml',
	'svgz' => 'image/svg+xml',
	
	// archives
	'zip' => 'application/zip',
	'rar' => 'application/x-rar-compressed',
	'exe' => 'application/x-msdownload',
	'msi' => 'application/x-msdownload',
	'cab' => 'application/vnd.ms-cab-compressed',
	
	// audio/video
	'mp3' => 'audio/mpeg',
	'qt' => 'video/quicktime',
	'mov' => 'video/quicktime',
	
	// adobe
	'pdf' => 'application/pdf',
	'psd' => 'image/vnd.adobe.photoshop',
	'ai' => 'application/postscript',
	'eps' => 'application/postscript',
	'ps' => 'application/postscript',
	
	// ms office
	'doc' => 'application/msword',
	'rtf' => 'application/rtf',
	'xls' => 'application/vnd.ms-excel',
	'ppt' => 'application/vnd.ms-powerpoint',
	
	// open office
	'odt' => 'application/vnd.oasis.opendocument.text',
	'ods' => 'application/vnd.oasis.opendocument.spreadsheet',

    // more at: http://www.w3schools.com/media/media_mimeref.asp
);

if (empty($_GET['file'])) {
    header('Location: '.BASE_ABS.'_admin/index.php?'.SID);
    ?>
    <html>
    <head>
        <title>Redirect to admin login</title>
    </head><body>
    You are not supposed to be here. 
    Please log in using the 
    <a href="<?php print(BASE_REL.'_admin/index.php'); ?>">admin interface</a>.
    </body></html>
    <?php
    exit;
}

$fileinfo = pathinfo($_GET['file']);

// Gibt es die Datei ueberhaupt?
if (!is_readable($fileinfo['basename'])) {
    header('HTTP/1.0 404 Not Found');
    echo '404 Not Found.';
    exit;
}
 
// Only logged in users can access "secret" files
if (empty($_SESSION['SID_login'])) {
    header('HTTP/1.0 403 Forbidden');
    echo '403 Forbidden.';
    exit;
}

$saveName = $fileinfo['basename'];
// If you need to remove the underscore sign, uncomment the following line
// $saveName = preg_replace('/^_(.*)/', '\1', $saveName);

if (array_key_exists(strtolower($fileinfo['extension']), $mime_types)) {
    header('Content-Type: ' . $mime_types[strtolower($fileinfo['extension'])]);
} else {
    header('Content-Type: application/octet-stream');
    header('Content-Disposition: attachment; filename="' . $saveName . '"');
}

readfile($fileinfo['basename']);