<?php
/*
+------------------------------------------------------------------------+
| ConPresso-Modification "Enhanced Parser"                               |
+------------------------------------------------------------------------+
| Copyright (c) Markus Rsel http://www.conpresso4.de                    |
| Developed for Gnther Ludwig http://www.amazingbytes.net               |
+------------------------------------------------------------------------+
| Authors: Markus Rsel <conpresso@gmx.de>                               |
+------------------------------------------------------------------------+

== Version: 4.1a / October 2020 ==

*/

if (!defined('CPO')) exit;

global $debugstring, $activeModules, $parsecount, $_REQUES, $translator;

if (!array_key_exists('parser',$activeModules)) {  // alten Parser nutzen, wenn Modul abgeschaltet wird
	include(CPO_BASEDIR.'_include/cpo_parser.php');
	return;
}

// prepare some vars
initVar($deleteifnotsetpart, NULL);
initVar($deleteiffirstpart, NULL);
initVar($deleteiflastpart, NULL);
initVar($deleteifnotfirstpart, NULL);
initVar($deleteifnotlastpart, NULL);
initVar($deleteifrubricfirstpart, NULL);
initVar($deleteifnotrubricfirstpart, NULL);
initVar($deleteifvaluepart, NULL);
initVar($deleteifsetpart, NULL);
initVar($parserMessage, NULL);
initVar($errorhandling, false);
initVar($debugging, false);
initVar($debugstring, '');
initVar($showTable, false);
initVar($showAllSteps, false);
initVar($showArrays, false);
initVar($TOTALPARSINGTIME, NULL);
initVar($PARSERRUNNINGSTART, NULL);

// einige Einstellungen fr die Fehlersuche
if (isset($_SESSION['SID_user']['admin']) && $_SESSION['SID_user']['admin']>=ROLE_SUPERUSER) {
	$errorhandling = ($activeModules['parser']['errorhandling']=="yes") ? true : false;  // aktiviert die Fehlermeldungen
	$debugging     = ($activeModules['parser']['debugging']=="yes")     ? true : false;  // aktiviert die Ausgabe der Templateverarbeitung
	$showTable     = ($activeModules['parser']['showTable']=="yes")     ? true : false;  // aktiviert die Templatestrukturanzeige
	$showAllSteps  = ($activeModules['parser']['showAllSteps']=="yes")  ? true : false;  // aktiviert die Detailanzeige jedes Parserdurchgangs
	$showArrays    = ($activeModules['parser']['showArrays']=="yes")    ? true : false;  // aktiviert die Anzeige der benutzen Arrays
	list($usec, $sec) = explode(' ', microtime());
	$PARSERRUNNINGSTART = (float)$usec + (float)$sec;
}

if ($debugging) $debuglink = '<div class="parserDebugging" name="debuglink' . $db_nr . '" onClick="DebugFenster'. $db_nr .'();" style="position:relative;padding:5px;height:24px;width:100%;background:#eeffee;cursor:pointer;"><span style="padding:5px;height:24px;background-image:url(../mod_parser/down_arrow.png);background-position:top left;background-repeat:no-repeat;"><span style="margin-left:30px;"><b>'.__('Debugging').'</b> ('.__('click to show').')</span></span> ';

global $tagOpenTypes, $tagCloseTypes;

if (!function_exists('parserHighlight'))  {
	function parserHighlight($code) {
		$tagOpenTypes = array (
							'file', 
							'html', 
							'image', 
							'text', 
							'url', 
							'value', 
							'slideshow', // mod_gallery
							'form', // mod_form
							'guestbook', // mod_guestbook
							'ifset', 
							'ifnotset', 
							'ifvalue', 
							'iffirst', 
							'iflast',
							'ifnotfirst', 
							'ifnotlast',
							'ifrubricfirst',
							'ifnotrubricfirst'
							);
		$tagCloseTypes = array (
							'ifSet', 
							'ifNotSet', 
							'ifvalue', 
							'iffirst', 
							'iflast',
							'ifnotfirst', 
							'ifnotlast',
							'ifrubricfirst',
							'ifnotrubricfirst'
							);
		$templateHighlighting = array('/&lt;!--\s*('.implode('|',$tagOpenTypes).')\s*:*.*?--&gt;/si', '/&lt;!--\s*\/('.implode('|',$tagCloseTypes).')\s*--&gt;/si');
		$templateHighlights = '<span style="color: #f00;">\0</span>';
		return preg_replace($templateHighlighting,$templateHighlights,nl2br(htmlspecialchars($code, ENT_COMPAT, 'ISO-8859-15')));
	}
}
if (!function_exists('showTable'))  {
	function showTable($parts,$levels,$title) {
		global $debugstring;
		$debugstring .= '<table border="0" cellpadding="0" cellspacing="0" style="position:relative;background:white;border:1px solid black;margin:5px 0;padding:5px;">';	
		$debugstring .= '<tr><td colspan="'.max($levels).'"><b>'.__('Structure table').' '.$title.':</b></td></tr>';
		for($s=0;$s<count($parts);$s++) {
			$debugstring .= '<tr>';
			for($z=0;$z<=$levels[$s];$z++) {
				$debugstring .= '<td style="background-color:#eeeeee;width:'.intval(100/(max($levels)+1)).'%">';
				if (trim($parts[$s]) == '')  {
					$debugstring .= ($levels[$s] == $z) ? str_replace ("\n", '<span style="color: #080">-- LF --</span><br />', $parts[$s]) : '&nbsp;';
				} else {
					$debugstring .= ($levels[$s] == $z) ? parserHighlight($parts[$s]) : '&nbsp;';
				}
				$debugstring .= '</td>';
			}
			$debugstring .= '</tr>';
		}
		$debugstring .= '</table>';	
	}
}
if (!function_exists('showTemplate'))  {
	function showTemplate($template_datei,$title,$bordercolor,$shadingcolor) {
		global $debugstring;
		$debugstring .= '<div style="position:relative;border:1px solid '.$bordercolor.';background-color:'.$shadingcolor.';margin:5px 0;padding:5px;"><b>'.$title.'</b><br />'.parserHighlight($template_datei).'</div>';	
	}
}

// Variablen initialisieren
$parsecount		= ($parsecount) ? $parsecount+1 : 1 ;
$parts			= array();
$types			= array();
$names			= array();
$levels			= array();
$contents		= array();
$opened			= array();
$openednot		= array();
$openedval		= array();
$warnings		= array();
$content		= array();
$replacer		= array();
$db_content		= new DB;
$fetchedcontent = array(); // stores which content type was fetched already, to minimize database queries

if (!isset($placeholders_cache)) $placeholders_cache = array();

// fetch settings for all placeholders of the active template
$placeholder = array();
if (!empty($templates_id)) {
    if (!array_key_exists($templates_id, $placeholders_cache)) {
        $placeholders_cache[$templates_id] = array();

        $dbq = new DB;
        $dbq->query('SELECT ttype, fieldname, ddata '
                   .'FROM '.CPO_PLACEHOLDERS.' '
                   .'WHERE templates_id='.(int)$templates_id);
        while($dbq->next_record()) {
            $phname = $dbq->v('fieldname').' ('.$typeNames[$dbq->v('ttype')].')';
            $placeholders_cache[$templates_id][$phname] = unserialize($dbq->v('ddata'));
        }
    }

    $placeholder = $placeholders_cache[$templates_id];
}

// create a regular expression with all available placeholders
$tagsPreg = '';
$_div = '';
foreach ($hdlr_parser_replacement as $k=>$v) {
    $tagsPreg .= $_div.preg_quote($k, '/');
    $_div = '|';
}
$tagsPreg = '/<!--\s*('.$tagsPreg.')\s*:\s*([^;]*?)\s*(;\s*(.*?)\s*)?-->/si';

// find and process all tags
preg_match_all($tagsPreg, $template_datei_full, $matches, PREG_SET_ORDER);
foreach ($matches as $match) {
    $_fullmatch = $match[0];
    $_type      = strtolower(trim($match[1]));
    $_name      = preg_replace("/(\r\n|\n|\r)/", ' ', trim($match[2]));
    $key        = $_name.' ('.$_type.')';
    $_params    = '';
    if (isset($match[4])) $_params = preg_replace("/(\r\n|\n|\r)/", ' ', trim($match[4]));

    // fetch content for this placeholder if not already done so
    if (!isset($content[$key])
        && array_key_exists($_type, $hdlr_parser_content)) {

        doActions($hdlr_parser_content[$_type]);
    }
    // if we don't have content for this placeholder, replace it with an empty string
    // FIXME is this necessary? Should it be done by parser_r_*? How much performance do we get?
    if (!array_key_exists($key, $content) || $content[$key]==false) {
        $replacer['/'.preg_quote($_fullmatch, '/').'/'] = '';
        continue;
    }

    // create an array with all parameters
    $parameters = array();
    foreach (explode(';', $_params) as $value) {
        $value = trim($value);
        if ($value=='') continue;

        // split 'a = b' or only 'a'
        preg_match('/\s*([^=]*)\s*(=(.*))?/', $value, $tmp);

        // no matches found?
        if (count($tmp)==0) continue;

        if (count($tmp)==2) { // only a flag
            $parameters[strtolower(trim($tmp[1]))] = true;
        }
        if (count($tmp)==4) { // a = b
            $parameters[strtolower(trim($tmp[1]))] = trim($tmp[3]);
        }
    }

    // create a replacement for the tag
    $replacement = '<!-- ERROR: unable to find replacement for the following placeholder (unknown type?): -->'.$_fullmatch;
    if (array_key_exists($_type, $hdlr_parser_replacement)) {
        doActions($hdlr_parser_replacement[$_type]);
    }

    // Experimental: FrontEnd-Editing ***************************************************************************************************************
    // if (file_exists(CPO_BASEDIR.'mod_feediting/includes/parser.inc.php')) include(CPO_BASEDIR.'mod_feediting/includes/parser.inc.php');
    // Experimental: FrontEnd-Editing ***************************************************************************************************************

    $replacer['/'.preg_quote($_fullmatch, '/').'/'] = $replacement;
}



preg_match_all('/<!--\s*(ifset|ifnotset|\/ifSet|\/ifNotSet|ifvalue|\/ifvalue|iffirst|\/iffirst|iflast|\/iflast|ifnotfirst|\/ifnotfirst|ifnotlast|\/ifnotlast|ifrubricfirst|\/ifrubricfirst|ifnotrubricfirst|\/ifnotrubricfirst)\s*:*\s*([^;]*?)\s*(;\s*(.*?)\s*)?-->/si', $template_datei, $matchpos, PREG_OFFSET_CAPTURE);
$lastPos=0;
$level=0;
$found=false;
for($m=0;$m<count($matchpos[0]);$m++) {
	if ($lastPos<$matchpos[0][$m][1]) {
		$parts[] = substr($template_datei,$lastPos,$matchpos[0][$m][1]-$lastPos);
		$types[] = '';
		$names[] = '';
		$levels[] = $level;
		$contents[] = $found;
	}
	//ab hier kommen die ifsets
	$parts[] = substr($template_datei,$matchpos[0][$m][1],strlen($matchpos[0][$m][0]));
	$type = strtolower($matchpos[1][$m][0]);
	$types[] = $type;
	$names[] = $matchpos[2][$m][0];
	if ($type == 'ifset' || $type == 'ifnotset' || $type == 'ifvalue' || $type == 'iffirst' || $type == 'iflast' || $type == 'ifnotfirst' || $type == 'ifnotlast' || $type == 'ifrubricfirst' || $type == 'ifnotrubricfirst') $level++;
	$levels[] = $level;
	if ($type == '/ifset' || $type == '/ifnotset' || $type == '/ifvalue' || $type == '/iffirst' || $type == '/iflast' || $type == '/ifnotfirst' || $type == '/ifnotlast' || $type == '/ifrubricfirst' || $type == '/ifnotrubricfirst') $level--;
	$found=false;

	if (strpos($matchpos[2][$m][0],'&&') !== false) {  // wir haben ein "und"
		$hits=array();
		$matchnames = explode('&&',$matchpos[2][$m][0]);
		foreach ($matchnames AS $matchname)  {
			foreach ($content as $key2=>$value2) { 
				if ($value2['name'] != trim($matchname)) continue 1;
				// we got content for this placeholder
				$hits[]=1;
			} // foreach
		} // foreach
		if ( (array_sum($hits) == count($matchnames) ) && count($matchnames) > 0 ) $found=true; // we got content for all these placeholders
	} elseif (strpos($matchpos[2][$m][0],'||') !== false) { // wir haben ein "oder"
		$matchnames = explode('||',$matchpos[2][$m][0]);
		foreach ($matchnames AS $matchname)  {
			foreach ($content as $key2=>$value2) { 
				if ($value2['name'] != trim($matchname)) continue 1;
				// we got content for this placeholder
				$found=true;   // einmal reicht ja...
			} // foreach	
		} // foreach
	} else {  // wir haben keines von beidem, also nur eine einfache Prfung
		foreach ($content as $key2=>$value2) { 
			if ($value2['name'] != $matchpos[2][$m][0]) continue;   //original
			// we got content for this placeholder
			$found=true;
		} // foreach
	}
	
	$contents[] = $found;
	$found=false;
	$lastPos=$matchpos[0][$m][1]+strlen($matchpos[0][$m][0]);
}
if ($lastPos<strlen($template_datei)) {
	$parts[] = substr($template_datei,$lastPos,strlen($template_datei)-$lastPos);
	$types[] = '';
	$names[] = '';
	$levels[] = $level;
	$contents[] = $found;
}

if ($debugging && $showArrays) {
	$debugstring .= '<pre style="position:relative;padding:5px;border:1px solid grey;background:#EFEFEF;">';
	$debugstring .= '$names-';
	$debugstring .= nl2br(htmlspecialchars(var_export($names,true), ENT_COMPAT, 'ISO-8859-15'));
	$debugstring .= "<br />".'$parts-';
	$debugstring .= nl2br(htmlspecialchars(var_export($parts,true), ENT_COMPAT, 'ISO-8859-15'));
	$debugstring .= "<br />".'$types-';
	$debugstring .= nl2br(htmlspecialchars(var_export($types,true), ENT_COMPAT, 'ISO-8859-15'));
	$debugstring .= "<br />".'$levels-';
	$debugstring .= nl2br(htmlspecialchars(var_export($levels,true), ENT_COMPAT, 'ISO-8859-15'));
	$debugstring .= '</pre>';
}

if ($debugging && $showTable) showTable($parts,$levels,__('complete template'));
if ($debugging)	showTemplate($template_datei,__('unparsed:'),'black','#eeeeee');

//if ($debugging)	echo '<div style="border:1px solid red;background-color:#FFDDDD;margin:5px 0;padding:5px;"><b>unparsed:</b><br /><div style="font-family:monospace;">'.parserHighlight($template_datei).'</div></div>';

// jetzt die Schleifen drehen
$maxlevel = (count($levels)>0) ? max($levels) : 0 ;
for ($if=$maxlevel;$if>0;$if--)  {
	for ($p=0;$p<count($parts);$p++)  {
		if ($levels[$p] != $if) continue;
//		echo $types[$p].'/'.$parsecount.'/'.$_REQUEST['pos'].'<br />';
		// ifsets *********************************************************************
		if ($types[$p] == 'ifset') {
//			if ($opened[$if]) $warnings[] = 'ifset: '.$names[$p].' not closed before reopening on same level!';
			$opened[$if] = $names[$p];
		}
		if ($types[$p] == 'ifset' && !$contents[$p]) {
			$parts[$p] = ''; $types[$p] = '';
			$deleteifsetpart = true;
		}
		if ($types[$p] == 'ifset' && $contents[$p]) {
			$parts[$p] = ''; $types[$p] = '';
		}
		$parts[$p] = ($deleteifsetpart) ? '' : $parts[$p];
		if ($types[$p] == '/ifset') {
			$parts[$p] = ''; $types[$p] = '';
			$deleteifsetpart = false;
//			if (!$opened[$if]) $warnings[] = 'trying to close an ifset that was not opened!';
			unset ($opened[$if]);
		}
		// ifnotsets *********************************************************************
		if ($types[$p] == 'ifnotset') {
//			if ($openednot[$if]) $warnings[] = 'ifnotset: '.$names[$p].' not closed before reopening on same level!';
			$openednot[$if] = $names[$p];
		}
		if ($types[$p] == 'ifnotset' && $contents[$p]) {
			$parts[$p] = ''; $types[$p] = '';
			$deleteifnotsetpart = true;
		}
		if ($types[$p] == 'ifnotset' && !$contents[$p]) {
			$parts[$p] = ''; $types[$p] = '';
		}
		$parts[$p] = ($deleteifnotsetpart) ? '' : $parts[$p];
		if ($types[$p] == '/ifnotset') {
			$parts[$p] = ''; $types[$p] = '';
			$deleteifnotsetpart = false;
//			if (!$openednot[$if]) $warnings[] = 'trying to close an ifnotset that was not opened!';
			unset ($openednot[$if]);
		}
		// iffirst/iflast *********************************************************************
		if ($types[$p] == 'iffirst' && $parsecount == 1) {
			$parts[$p] = ''; $types[$p] = '';
		}
		if ($types[$p] == 'iffirst' && $parsecount > 1) {
			$parts[$p] = ''; $types[$p] = '';
			$deleteiffirstpart = true;
		}
		$parts[$p] = ($deleteiffirstpart) ? '' : $parts[$p];
		if ($types[$p] == '/iffirst') {
			$parts[$p] = ''; $types[$p] = '';
			$deleteiffirstpart = false;
		}
		if ($types[$p] == 'iflast' && $parsecount == $anzahl) {
			$parts[$p] = ''; $types[$p] = '';
		}
		if ($types[$p] == 'iflast' && $parsecount < $anzahl) {
			$parts[$p] = ''; $types[$p] = '';
			$deleteiflastpart = true;
		}
		$parts[$p] = ($deleteiflastpart) ? '' : $parts[$p];
		if ($types[$p] == '/iflast') {
			$parts[$p] = ''; $types[$p] = '';
			$deleteiflastpart = false;
		}
		// ifnotfirst/ifnotlast *********************************************************************
		if ($types[$p] == 'ifnotfirst' && $parsecount > 1) {
			$parts[$p] = ''; $types[$p] = '';
		}
		if ($types[$p] == 'ifnotfirst' && $parsecount == 1) {
			$parts[$p] = ''; $types[$p] = '';
			$deleteifnotfirstpart = true;
		}
		$parts[$p] = ($deleteifnotfirstpart) ? '' : $parts[$p];
		if ($types[$p] == '/ifnotfirst') {
			$parts[$p] = ''; $types[$p] = '';
			$deleteifnotfirstpart = false;
		}
		if ($types[$p] == 'ifnotlast' && $parsecount < $anzahl) {
			$parts[$p] = ''; $types[$p] = '';
		}
		if ($types[$p] == 'ifnotlast' && $parsecount == $anzahl) {
			$parts[$p] = ''; $types[$p] = '';
			$deleteifnotlastpart = true;
		}
		$parts[$p] = ($deleteifnotlastpart) ? '' : $parts[$p];
		if ($types[$p] == '/ifnotlast') {
			$parts[$p] = ''; $types[$p] = '';
			$deleteifnotlastpart = false;
		}
		// ifrubricfirst *********************************************************************
		if ($types[$p] == 'ifrubricfirst' && $parsecount == 1 && $_REQUEST['pos'] == 0) {
			$parts[$p] = ''; $types[$p] = '';
		}
		if ($types[$p] == 'ifrubricfirst' && ($parsecount > 1 || $_REQUEST['pos'] != 0)) {
			$parts[$p] = ''; $types[$p] = '';
			$deleteifrubricfirstpart = true;
		}
		$parts[$p] = ($deleteifrubricfirstpart) ? '' : $parts[$p];
		if ($types[$p] == '/ifrubricfirst') {
			$parts[$p] = ''; $types[$p] = '';
			$deleteifrubricfirstpart = false;
		}
		// ifnotrubricfirst *********************************************************************
		if ($types[$p] == 'ifnotrubricfirst' && ($parsecount > 1 || $_REQUEST['pos'] != 0)) {
			$parts[$p] = ''; $types[$p] = '';
		}
		if ($types[$p] == 'ifnotrubricfirst' && $parsecount == 1 && $_REQUEST['pos'] == 0) {
			$parts[$p] = ''; $types[$p] = '';
			$deleteifnotrubricfirstpart = true;
		}
		$parts[$p] = ($deleteifnotrubricfirstpart) ? '' : $parts[$p];
		if ($types[$p] == '/ifnotrubricfirst') {
			$parts[$p] = ''; $types[$p] = '';
			$deleteifnotrubricfirstpart = false;
		}
		// ifvalue *********************************************************************
		if ($types[$p] == 'ifvalue') {
//			echo $names[$p];
			$eq = false;
			if (strstr ($names[$p],"!=") && !$eq) $eq = "!=";
			if (strstr ($names[$p],">=") && !$eq) $eq = ">=";
			if (strstr ($names[$p],"<=") && !$eq) $eq = "<=";
			if (strstr ($names[$p],"=")  && !$eq) $eq = "=";
			if (strstr ($names[$p],">")  && !$eq) $eq = ">";
			if (strstr ($names[$p],"<")  && !$eq) $eq = "<";
			$expr = explode($eq,$names[$p]);
//			$names[$p] = trim($expr[0]);
//			print_r($expr);
			foreach ($content as $key2=>$value2) {
				if ($value2['name'] != trim($expr[0])) continue;
				$valcontent = $value2['text'];
//				echo $value2['name'];
			}
			$eq_ok=false;
			switch ($eq)  {
				case "!=": if (trim($valcontent) != trim($expr[1])) $eq_ok=true; break;
				case ">=": if (trim($valcontent) >= trim($expr[1])) $eq_ok=true; break;
				case "<=": if (trim($valcontent) <= trim($expr[1])) $eq_ok=true; break;
				case "=":  if (trim($valcontent) == trim($expr[1])) $eq_ok=true; break;
				case ">":  if (trim($valcontent) >  trim($expr[1])) $eq_ok=true; break;
				case "<":  if (trim($valcontent) <  trim($expr[1])) $eq_ok=true; break;
			}
			$parts[$p] = ''; $types[$p] = '';
			if (!$eq_ok) $deleteifvaluepart = true;
//			if ($openedval[$if]) $warnings[] = 'ifvalue: '.$names[$p].' not closed before reopening on same level!';
			$openedval[$if] = trim($expr[0]);
		}
		$parts[$p] = ($deleteifvaluepart) ? '' : $parts[$p];
		if ($types[$p] == '/ifvalue') {
			$parts[$p] = ''; $types[$p] = '';
			$deleteifvaluepart = false;
//			if (!$openedval[$if]) $warnings[] = 'trying to close an ifvalue that was not opened!';
			unset ($openedval[$if]);
		}
		$levels[$p]--;
	}
	if ($debugging && $showAllSteps)  showTemplate(implode($parts),__('Level').' '.$if.' '.__('parsed:'),'red','#FFDDDD');
	if ($debugging && $showTable && $showAllSteps)  showTable($parts,$levels,__('parsed to level ').max($levels));
//	if ($debugging && $showAllSteps)	echo '<div style="border:1px solid red;background-color:#FFDDDD;margin:5px 0;padding:5px;"><b>Level '.$if.' parsed:</b><br /><div style="font-family:monospace;">'.parserHighlight(implode($parts)).'</div></div>';
}


$template_datei = implode($parts);

if ($debugging)  showTemplate($template_datei,__('conditionally parsed:'),'blue','#DDDDFF');
//if ($debugging)	echo '<div style="border:1px solid blue;background-color:#DDDDFF;margin:5px 0;padding:5px;"><b>conditionally parsed:</b><br /><div style="font-family:monospace;">'.parserHighlight($template_datei).'</div></div>';



// *******************************************************************************************************************
// start errorhandling: Ausgabe der Meldungen der Prfung auf falsche Verschachtelung
if ($errorhandling)
{
	if ($level != 0)					$parserMessage[] = 'incomplete ifset-sequence, e.g. too many opening or closing tags!'; 
	foreach ($opened AS $ph_name)		$parserMessage[] = 'ifset: '.$ph_name.' not closed!';       
	foreach ($openednot AS $ph_name)	$parserMessage[] = 'ifnotset: '.$ph_name.' not closed!';       
	foreach ($openedval AS $ph_name)	$parserMessage[] = 'ifvalue: '.$ph_name.' not closed!';       
	foreach ($warnings AS $warning)		$parserMessage[] = $warning;       
	if ($parserMessage) { 
	   $parserMessage[] = '(the number of messages does not represent the number of faults in your template, it represents the number of errors caused by your faults, e.g. one single fault can cause several errors)'; 
	   $message = $parserMessage; 
	   include(CPO_BASEDIR.'_include/message.php'); 
	   unset ($parserMessage); 
	}
}
// end errorhandling
// *******************************************************************************************************************



// Content einsetzen
if (count($replacer)>0)
    $template_datei = preg_replace(array_keys($replacer), array_values($replacer), $template_datei);

// end parsing
// *******************************************************************************************************************

if ($debugging)  showTemplate($template_datei,__('parsed and replaced:'),'green','#DDFFDD');
//if ($debugging)	echo '<div style="border:1px solid green;background-color:#DDFFDD;margin:5px 0;padding:5px;"><b>parsed and replaced:</b><br /><div style="font-family:monospace;">'.parserHighlight($template_datei).'</div></div>';

//if ($debugging) $debugstring .= '</div>';

// post parsing ******************************************************************************************************

// Imagetool-Mod 
if (array_key_exists('Imagetool', $activeModules)) include(CPO_BASEDIR . $activeModules['Imagetool']['directory'] . '/includes/click_images.inc.php'); 

if (trim($activeModules['parser']['postparsing']!=''))  {
	$postparsingActions = explode("\n",trim($activeModules['parser']['postparsing']));
	foreach($postparsingActions AS $postparsingAction)  {
		if (file_exists(CPO_BASEDIR.trim($postparsingAction)) && is_readable(CPO_BASEDIR.trim($postparsingAction))) include (CPO_BASEDIR.trim($postparsingAction));
	}
}
if ($debugging)  showTemplate($template_datei,__('After post parsing:'),'green','#DDDDFF');
if ($debugging)  {
	list($usec, $sec) = explode(' ', microtime());
	$thisTurnTime = (float)$usec + (float)$sec - $PARSERRUNNINGSTART;
    $_line = sprintf('<b>%s:</b> %8.4fs', __('Parsing this article'), $thisTurnTime);
	$TOTALPARSINGTIME += $thisTurnTime;
    $_line .= sprintf(' <b>%s:</b> %8.4fs', __('Total parsing time'), $TOTALPARSINGTIME);
	$debuglink .= '<span style="padding:5px;height:24px;background-color:#eeffee;background-image:url(../mod_parser/stopwatch.gif);background-position:top left;background-repeat:no-repeat;"><span style="margin-left:30px;">'.$_line.'</span></span></div>';
}

if ($debugging)  {
	echo $debuglink;

?>
<script type="text/javascript">
function DebugFenster<?php echo $db_nr; ?> () {
  ImageFenster = window.open('', "Degugfenster<?php echo $db_nr; ?>", "width=600,height=700,resizable=yes,scrollbars=yes");
  ImageFenster.focus();
  ImageFenster.document.open();
  with (ImageFenster) {
	  document.write("<html><head><title>Parser Debugging by mod_parser</title></head><body>");
<?php

$string = explode("\n", $debugstring);
foreach ($string as $output) {
    if ($output=='') continue;

    // if output is for javascript, clean "up"
    $output = preg_replace("/(\015\012)|(\015)|(\012)/", '', $output);
    $output = str_replace("'", "\\'", $output);
    printf("	  document.write('%s');\n", $output);
}

?>	
	  document.write("</body></html>");
	  ImageFenster.document.close();
  }
}
</script>
<?php
}
?>