<?php /* _\|/_              vim: set et ts=4 sw=4 sts=4 nowrap fdm=marker:
         (o o)                         
+-----oOO-{_}-OOo--------------------------------------------------------+
| ConPresso - Foolproof Content Management                               |
+------------------------------------------------------------------------+
| Copyright (c) Bartels Schoene GmbH & Co. KG, 48149 Muenster, Germany   |
+------------------------------------------------------------------------+
| This source file is subject to ConPresso license version 2002-11-16    |
| that is bundled with this package in the files LICENSE_EN.TXT or       |
| LICENSE_DE.TXT, and is available through the world-wide-web at         |
|     http://www.conpresso.de/license/LICENSE_EN-2002-11-16.TXT or       |
|     http://www.conpresso.de/license/LICENSE_DE-2002-11-16.TXT          |
|                                                                        |
| If you did not receive a copy of the ConPresso license and are unable  |
| to obtain it through the world-wide-web, please send a note to         |
| <license@conpresso.de> so we can e-mail you a copy immediately.        |
+------------------------------------------------------------------------+
| Authors: ConPresso development and support team <support@conpresso.de> |
+------------------------------------------------------------------------+
  $Id$ */
define('CPO', true);
define('SETUP', true);
require_once('../_include/common.inc.php');

session_start(); // needed to get code

// preparing variables
$errors = array();
$message = array();

if (!isset($_POST['DB_hostname']))     $_POST['DB_hostname'] = 'localhost';
if (!isset($_POST['DB_name']))         $_POST['DB_name']     = '';
if (!isset($_POST['DB_login']))        $_POST['DB_login']    = '';
if (!isset($_POST['DB_password']))     $_POST['DB_password'] = '';
if (!isset($_POST['DB_prefix']))       $_POST['DB_prefix']   = 'cpo4';

if (!isset($_POST['cpo_project']))     $_POST['cpo_project']    = '';
if (!isset($_POST['Admin_login']))     $_POST['Admin_login']    = 'admin';
if (!isset($_POST['Admin_password']))  $_POST['Admin_password'] = '';
if (!isset($_POST['Admin_password2'])) $_POST['Admin_password2'] = '';
if (!isset($_POST['Admin_email']))     $_POST['Admin_email']    = $_POST['email'] ;
if (!isset($_POST['cpo_email']))       $_POST['cpo_email']      = $_POST['email'] ;
if (!isset($_POST['base_URL']))        $_POST['base_URL']       = '';

if (!isset($_POST['action']))          $_POST['action']       = '';

switch($_POST['action']) {
case 'call form':
    $ok = true;

    if (!isset($_SESSION['instcode'])) {
        $_SESSION['message'] = ___('You missed to send the mail to get the required code.');
        $errors['Admin_email'] = ___('You missed to send the mail to get the required code.');
    }
    if (isset($_POST['code']) && isset($_SESSION['instcode']) && ($_POST['code']) != $_SESSION['instcode']) {
        $_SESSION['message'] = ___('The entered installation code is wrong.');
        $errors['Admin_email'] = ___('The entered installation code is wrong.');
    }
    if (empty($_POST['code'])) {
        $_SESSION['message'] = ___('Please enter the installation code sent by e-mail.');
        $errors['Admin_email'] = ___('Please enter the installation code sent by e-mail.');
    }
    if (count($errors)>0) {
        $_SESSION['language'] = $_POST['language'];
        $_SESSION['email']    = $_POST['email'];
        header('location:step1.php');
        exit;
    }
    break;

case 'submit form':
    $ok = true;

    if (!preg_match('=^[-a-z0-9_]+$=i', $_POST['DB_prefix'])) {
        $message[] = ___('Please make sure that the prefix only contain letters (\'a\'-\'z\'), numbers (\'0\'-\'9\') and the underscore (\'_\').');
        $errors['prefix'] = ___('Please make sure that the prefix only contain letters (\'a\'-\'z\'), numbers (\'0\'-\'9\') and the underscore (\'_\').');
    }

    if (empty($_POST['DB_name'])) {
        $message[] = ___('Please enter a database name.');
        $errors['DB_name'] = ___('Please enter a database name.');
        $ok = false;
    }
    
    if ($ok && !$mysqli=@mysqli_connect($_POST['DB_hostname'], $_POST['DB_login'], $_POST['DB_password'])) {
        $message[] = ___('Unable to connect to the database. Please check database hostname, login and password.');
        $errors['connect']  = ___('Unable to connect to the database. Please check database hostname, login and password.');
        $ok = false;
    }
    if ($ok && !mysqli_select_db($mysqli, $_POST['DB_name'])) {
        $message[] = ___('Unable to select the database. Please check database name, login and password. Also check if the database exists.');
        $errors['select'] = ___('Unable to select the database. Please check database name, login and password. Also check if the database exists.');
        $ok = false;
    }

    // FIXME check if database exists before?
    if ($ok && !@mysqli_query($mysqli, sprintf('CREATE TABLE %s_block (id int(11))', $_POST['DB_prefix']))) {
        $message[] = sprintf(___('Unable to create a table in the database. Please check database name, login and password. Also make sure there are no %s tables with the same prefix yet.'), htmlspecialchars($CPO['product'], ENT_COMPAT, 'ISO-8859-15'));
        $errors['CREATE'] = sprintf(___('Unable to create a table in the database. Please check database name, login and password. Also make sure there are no %s tables with the same prefix yet.'), htmlspecialchars($CPO['product'], ENT_COMPAT, 'ISO-8859-15'));
    } else {
        if ($ok) @mysqli_query($mysqli, sprintf('DROP TABLE %s_block', $_POST['DB_prefix']));
    }

    if (empty($_POST['Admin_login'])) {
        $message[] = ___('Please enter a login name for the administrative account.');
        $errors['Admin_login'] = ___('Please enter a login name for the administrative account.');
    }

    if (empty($_POST['Admin_password'])) {
        $message[] = ___('Please enter a password for the administrative account.');
        $errors['Admin_password'] = ___('Please enter a password for the administrative account.');
    }

    if ($_POST['Admin_password']!=$_POST['Admin_password2']) {
        $message[] = ___('Please make sure the administrativ account password and the password confirmation match.');
        $errors['Admin_password'] = ___('Please make sure the administrativ account password and the password confirmation match.');
    }

    if (empty($_POST['Admin_email'])) {
        $message[] = ___('Please enter an e-mail address for the administrative account.');
        $errors['Admin_email'] = ___('Please enter an e-mail address for the administrative account.');
    }

    if (empty($_POST['cpo_project'])) {
        $message[] = sprintf(___('Please enter a name for this %s-project.'), htmlspecialchars($CPO['product'], ENT_COMPAT, 'ISO-8859-15'));
        $errors['cpo_project'] = sprintf(___('Please enter a name for this %s-project.'), htmlspecialchars($CPO['product'], ENT_COMPAT, 'ISO-8859-15'));
    }

    if (empty($_POST['cpo_email'])) {
        $message[] = sprintf(___('Please enter an e-mail address for %s system e-mails.'), htmlspecialchars($CPO['product'], ENT_COMPAT, 'ISO-8859-15'));
        $errors['cpo_email'] = sprintf(___('Please enter an e-mail address for %s system e-mails.'), htmlspecialchars($CPO['product'], ENT_COMPAT, 'ISO-8859-15'));
    }

    if (count($errors)<=0) {
        $dbFileContent = '<?php'."\n".
                         '// '.'NOTE: Please make sure there are no blanks '."\n".
                         '// '.'or empty lines in front of "<? php" or after "? >"!'."\n\n".
                         '$dbPrefix = \''.$_POST['DB_prefix'].'\';'."\n".
                         'class DB extends DB_mysql {'."\n".
                         '    var $Host     = \''.$_POST['DB_hostname'].'\';'."\n".
                         '    var $Database = \''.$_POST['DB_name'].'\';'."\n".
                         '    var $User     = \''.$_POST['DB_login'].'\';'."\n".
                         '    var $Password = \''.$_POST['DB_password'].'\';'."\n".
                         '}'."\n".
                         '?>'."\n";

        // File does not exist, create it or make the user do so.
        if (!@file_exists(CPO_BASEDIR.'_cfg/db.inc.php')){
            // check if we can write the database config file
            $check = @fopen(CPO_BASEDIR.'_cfg/db.inc.php', 'w');
            if (!$check) {
                $message[] = sprintf(___(
                    'Unable to create the database configuration file %s. Please allow write access to the file, so that the setup can create it automatically or create the file yourself using the following content:').'<br /><pre>'.htmlspecialchars($dbFileContent, ENT_COMPAT, 'ISO-8859-15').'</pre>',
                    '<em>_cfg/db.inc.php</em>'
                    );
                $errors['createfile'] = sprintf(___(
                    'Unable to create the database configuration file %s. Please allow write access to the file, so that the setup can create it automatically or create the file yourself using the following content:').'<br /><pre>'.htmlspecialchars($dbFileContent, ENT_COMPAT, 'ISO-8859-15').'</pre>',
                    '<em>_cfg/db.inc.php</em>'
                    );

            } else {
                if (!fwrite($check, $dbFileContent)) {
                    $message[] = sprintf(___(
                        'There were problems creating the database configuration file %s. Please check if write access is enabled to the directory, the file does already exist, etc.'), 
                        '<em>_cfg/db.inc.php</em>'
                    );
                    $errors['createfile'] = sprintf(___(
                        'There were problems creating the database configuration file %s. Please check if write access is enabled to the directory, the file does already exist, etc.'), 
                        '<em>_cfg/db.inc.php</em>'
                    );
                }
                fclose($check);
            }
        }

        // File exists, check if settings are OK
        if (@file_exists(CPO_BASEDIR.'_cfg/db.inc.php')){
            if (!is_readable(CPO_BASEDIR.'_cfg/db.inc.php')) {
                $message[] = sprintf(___('Please make sure the file %s is readable.'), '<em>_cfg/db.inc.php</em>');
                $errors['dbfile'] = sprintf(___('Please make sure the file %s is readable.'), '<em>_cfg/db.inc.php</em>');
            } else {
                require_once(CPO_BASEDIR.'_include/db_mysql.inc.php');
                include_once(CPO_BASEDIR.'_cfg/db.inc.php');
                $dbTest = new DB;
                if ($_POST['DB_hostname']!=$dbTest->Host) {
                    $message[] = sprintf(___('Please make sure the database hostname you entered and the hostname included in %s are the same.'), '<em>_cfg/db.inc.php</em>');
                    $errors['dbhostname'] = sprintf(___('Please make sure the database hostname you entered and the hostname included in %s are the same.'), '<em>_cfg/db.inc.php</em>');
                }
                if ($_POST['DB_login']!=$dbTest->User) {
                    $message[] = sprintf(___('Please make sure the database login you entered and the login included in %s are the same.'), '<em>_cfg/db.inc.php</em>');
                    $errors['dblogin'] = sprintf(___('Please make sure the database login you entered and the login included in %s are the same.'), '<em>_cfg/db.inc.php</em>');
                }
                if ($_POST['DB_password']!=$dbTest->Password) {
                    $message[] = sprintf(___('Please make sure the database password you entered and the password included in %s are the same.'), '<em>_cfg/db.inc.php</em>');
                    $errors['dbpassword'] = sprintf(___('Please make sure the database password you entered and the password included in %s are the same.'), '<em>_cfg/db.inc.php</em>');
                }
                if ($_POST['DB_name']!=$dbTest->Database) {
                    $message[] = sprintf(___('Please make sure the database name you entered and the name included in %s are the same.'), '<em>_cfg/db.inc.php</em>');
                    $errors['dbname'] = sprintf(___('Please make sure the database name you entered and the name included in %s are the same.'), '<em>_cfg/db.inc.php</em>');
                }
                if ($dbPrefix!=$_POST['DB_prefix']) {
                    $message[] = sprintf(___('Please make sure the database prefix you entered and the prefix included in %s are the same.'), '<em>_cfg/db.inc.php</em>');
                    $errors['dbname'] = sprintf(___('Please make sure the database prefix you entered and the prefix included in %s are the same.'), '<em>_cfg/db.inc.php</em>');
                }
            }

            if (count($errors)<=0) {
                require_once('step3.inc.php');
                return;
            }
        }
    }
    break;
}

$notimeout = true;
include(CPO_BASEDIR.'_include/header.php');
?>
<script type="text/javascript">
    parent.cpo_navigation.location.href = "../_admin/frame_navigation.php?s=2&l=<?php echo urlencode($_POST['language']); ?>";
</script>
<?php
printPageHeader(
    ___('Step 2/3 - Settings'), 
    sprintf(___('This is step two of three during the %s installation. On this page you\'ll need to enter some information like database server, login and password and others. This data is needed to finish the installation in step three.'), htmlspecialchars($CPO['product'], ENT_COMPAT, 'ISO-8859-15')),
    'setup-setupcall.html#setup-step2',
    false, '', $_POST['language']
);

if (count($message)>0) {
    print('<div style="border: 1px solid #c00;"><ul>');
    foreach ($message as $v) printf('<li>%s</li>', $v);
    print('</ul></div>'); 
} 

print('<form method="post" action="step2.php">');
print('<input type="hidden" name="action" value="submit form">');
printf('<input type="hidden" name="language" value="%s">', htmlspecialchars($_POST['language'], ENT_COMPAT, 'ISO-8859-15'));
print('<p>'.sprintf(___('%s needs a database server and a login/password combination to access a database. This database usually is created by your server provider. If you don\'t have this data yet, please contact the administrator of the server. %s also needs to be able to create database tables in the database.'), htmlspecialchars($CPO['product'], ENT_COMPAT, 'ISO-8859-15'), htmlspecialchars($CPO['product'], ENT_COMPAT, 'ISO-8859-15')).'</p><br />');
print('<table class="colored" style="width: 100%;"><thead><tr><th colspan="2"><span>'.___('Database settings').'</span></th></tr></thead><tbody>');
printf('<tr><td class="leftrow">'.___('Hostname of server:').'</td><td class="rightrow"><input type="text" class="text inputwidth" name="DB_hostname" value="%s">%s</td></tr>',
    htmlspecialchars($_POST['DB_hostname'], ENT_COMPAT, 'ISO-8859-15'),
    help('setup-setupcall.html#setup-step2-databaseserver', false, '', $_POST['language'])
);
printf('<tr><td>'.___('Database name:').'</td><td><input type="text" class="text inputwidth" name="DB_name" value="%s">%s</td></tr>',
    htmlspecialchars($_POST['DB_name'], ENT_COMPAT, 'ISO-8859-15'),
    help('setup-setupcall.html#setup-step2-databasename', false, '', $_POST['language'])
);
printf('<tr><td>'.___('Database login:').'</td><td><input type="text" class="text inputwidth" name="DB_login" value="%s">%s</td></tr>',
    htmlspecialchars($_POST['DB_login'], ENT_COMPAT, 'ISO-8859-15'),
    help('setup-setupcall.html#setup-step2-databaselogin', false, '', $_POST['language'])
);
printf('<tr><td>'.___('Database password (shown):').'</td><td><input type="text" class="text inputwidth" name="DB_password" value="%s">%s</td></tr>', 
    htmlspecialchars($_POST['DB_password'], ENT_COMPAT, 'ISO-8859-15'),
    help('setup-setupcall.html#setup-step2-databasepassword', false, '', $_POST['language'])
);

printf('<tr><td class="leftrow">'.___('Table prefix:').'</td><td class="rightrow">'
    .sprintf(___('To allow multiple %s installations to share one database you can use different prefixes for the names of the tables. Please enter a prefix below. Because of restrictions of the database the prefix may only contain letters (\'a\'-\'z\'), numbers (\'0\'-\'9\') and the underscore (\'_\')'), htmlspecialchars($CPO['product'], ENT_COMPAT, 'ISO-8859-15'))
    .'<input type="text" class="text inputwidth" name="DB_prefix" value="%s">%s</td></tr>',
    htmlspecialchars($_POST['DB_prefix'], ENT_COMPAT, 'ISO-8859-15'),
    help('setup-setupcall.html#setup-step2-tableprefix', false, '', $_POST['language'])
);
print('</tbody></table>');

print('<p>'.sprintf(___('The following settings are crucial to %s. They contain the absolute basic settings to get %s running.'), htmlspecialchars($CPO['product'], ENT_COMPAT, 'ISO-8859-15'), htmlspecialchars($CPO['product'], ENT_COMPAT, 'ISO-8859-15')).'</p>');
print(___('Administrator login data is needed to create a basic user account with all rights.').'</p><br />');
print('<table class="colored" style="width: 100%;"><thead><tr><th colspan="2"><span>'.___('Sytem settings').'</span></th></tr></thead><tbody>');
print('<tr><td class="leftrow">'.___('Name of project:').'</td><td class="rightrow">');
printf('<input type="text" class="text inputwidth" name="cpo_project" value="%s">%s</td></tr>',
    htmlspecialchars($_POST['cpo_project'], ENT_COMPAT, 'ISO-8859-15'),
    help('setup-setupcall.html#setup-step2-projectname', false, '', $_POST['language'])
);

printf('<tr><td class="leftrow">'.___('Administrator login:').'</td><td class="rightrow"><input type="text" class="text inputwidth" name="Admin_login" value="%s">%s</td></tr>',
    htmlspecialchars($_POST['Admin_login'], ENT_COMPAT, 'ISO-8859-15'),
    help('setup-setupcall.html#setup-step2-administratorlogin', false, '', $_POST['language'])
);
printf('<tr><td>'.___('Administrator password (not shown):').'</td><td><input type="password" class="text inputwidth" name="Admin_password" value="%s">%s</td></tr>',
    htmlspecialchars($_POST['Admin_password'], ENT_COMPAT, 'ISO-8859-15'),
    help('setup-setupcall.html#setup-step2-administratorlogin', false, '', $_POST['language'])
);
printf('<tr><td>'.___('Password confirmation (not shown):').'</td><td><input type="password" class="text inputwidth" name="Admin_password2" value="%s">%s</td></tr>', 
    htmlspecialchars($_POST['Admin_password2'], ENT_COMPAT, 'ISO-8859-15'),
    help('setup-setupcall.html#setup-step2-administratorlogin', false, '', $_POST['language'])
);
printf('<tr><td>'.___('Administrator e-mail:').'</td><td><input type="text" class="text inputwidth" name="Admin_email" value="%s">%s</td></tr>',
    htmlspecialchars($_POST['Admin_email'], ENT_COMPAT, 'ISO-8859-15'), 
    help('setup-setupcall.html#setup-step2-administratoremail', false, '', $_POST['language'])
);

print('<tr><td class="leftrow">'.___('System e-mail:').'</td><td class="rightrow">');
print(sprintf(___('The system e-mail address is used for all e-mails sent from %s.'), htmlspecialchars($CPO['product'], ENT_COMPAT, 'ISO-8859-15')));
printf('<input type="text" class="text inputwidth" name="cpo_email" value="%s">%s</td></tr>',
    htmlspecialchars($_POST['cpo_email'], ENT_COMPAT, 'ISO-8859-15'),
    help('setup-setupcall.html#setup-step2-systememail', false, '', $_POST['language'])
);

print('<tr><td class="leftrow">'.___('Base-URL:').'</td>');
print('<td class="rightrow">');
print(sprintf(___('The base-URL is needed internally to create links to %s rubrics, images, etc. %s can try to determine it itself. In this case, leave the field empty or enter \'auto\'.'), htmlspecialchars($CPO['product'], ENT_COMPAT, 'ISO-8859-15'), htmlspecialchars($CPO['product'], ENT_COMPAT, 'ISO-8859-15')));
printf('<input type="text" class="text inputwidth" name="base_URL" value="%s">%s',
    htmlspecialchars($_POST['base_URL'], ENT_COMPAT, 'ISO-8859-15'),
    help('setup-setupcall.html#setup-step2-baseaddress', false, '', $_POST['language'])
);
print('<br /><img src="'.SKIN_URL.'sym_note.gif" style="vertical-align: text-bottom;" alt="'.___('Note').'" title="'.___('Note').'"> ');
print(sprintf(___('In case of an empty field or \'auto\' %s will use: '), htmlspecialchars($CPO['product'], ENT_COMPAT, 'ISO-8859-15')).'<em>'.htmlspecialchars($auto_base_url, ENT_COMPAT, 'ISO-8859-15').'</em><br />');
print(___('Please make sure that this value is correct before using the autodetected value!'));
print('</td></tr>');
print('</tbody></table>');


print('<h1>'.___('Summary').'</h1>');
if (count($errors)>0) {
    print(___('Some problems were discovered while testing the entered settings.  Before you go on and submit the form again, fix the following problems:'));
    print('<ul>');
    foreach ($errors as $v) print('<li>'.$v.'</li>');
    print('</ul>');
    print(___('To check your settings again and proceed to step three, please'));
} else {
    print(___('To check your settings and proceed to step three, please'));
}
printf(' <input type="submit" class="button" value="%s">', ___('submit the form'));
print('</form>');

echo '<br /><br />';
include(CPO_BASEDIR.'_include/footer.php'); 
?>
